# Spanish translation style guide

Use informal Spanish for translation:

* Informal "you" (*tú*) instead of formal form *usted*. Many top software
  companies (e.g. Google) use the informal one, because it's much more common in
  the daily language and avoids making translations look like they were written
  by machines.

* Imperative, active, and continuous verbs, e.g. *manage streams* -
  *gestionar canales*, not *gestión de canales*.

* Not using reflexive *se* e.g. *log out* should be *salir*, not *salirse*,
  whenever the infinitive form is possible without making the translation
  awkward.

* Warm and friendly phrasing whenever appropriate.

* No slang or regional phrases that could be unclear or too informal.

* Balance common verbs and nouns with specific IT-related translations
  of English terms - this can be tricky, try to check how other
  resources were translated (e.g. Gmail, Microsoft websites, Facebook)
  to decide what wouldn't sound awkward / rude in Spanish.

* Latest RAE rule ("solo" should
  [**never**](https://www.rae.es/consultas/el-adverbio-solo-y-los-pronombres-demostrativos-sin-tilde)
  have accent, even when it can be replaced with "solamente").

Some terms are very tricky to translate, so be sure to communicate
with other Spanish speakers in the community. It's all about making
Zulip friendly and usable.

## Términos
* Message - **Mensaje**
* Private message (PM) - **Mensaje privado (MP)**
* Group PM - **mensaje privado grupal**: even though "MP grupal" is the most
  precise translation, preferably don't use that one. Many users may not
  associate "MP" with *private message* in a group context. Better use it
  without abbreviations.
* Stream - **Canal**: the use of the literal Spanish word for stream
  "Flujo" is very confusing and not the correct metaphor for Spanish
  speaking people. The correct term would be "canal" (*channel*).
* Topic - **Tema**
* Private/invite-only stream - **Canal privado/limitado por invitación**: (lit.
  *channel limited by invitation*)
* Public stream - **Canal público**
* Bot - **Bot**
* Embedded bot - **Bot integrado**
* Interactive bot - **Bot interactivo**
* Integration - **Integración**
* Notification - **Notificación**
* Alert word - **Alerta**: this is only *alert*. Nonetheless, adding *word* may
  make the term confusing (something like *danger!* could be a "palabra de
  alerta" as well). Google Alerts uses "alerta" in its Spanish translation.
* View - **Vista**
* Filter - **Filtro**: as used with narrowing (see below).
* Home - **Inicio**: we never use the term "Hogar" (literally home) in Spanish.
* Emoji - **Emoticono** (plural: **emoticonos**)
* Slash command - **/comando**
* Webhook - **Webhook**
* Endpoint - **Endpoint**

## Frases
* Subscribe/Unsubscribe to a stream - **Suscribir a/Desuscribir de un canal**
* Narrow to - **Filtrar solo**: this is *filter only*, because there's no other
  word that's common enough in Spanish for *to narrow* except for "filtrar".
* Mute/Unmute - **Silenciar/No silenciar**
* Deactivate/Reactivate - **Desactivar/Reactivar**
* Search - **Buscar**
* Pin - **Fijar** (lit. *to fixate*)
* Mention/@mention - **Mencionar/@mención**
* Invalid - **Inválido**
* Customization - **Personalización**
* I want - **Yo quiero**
* User - **Usuario**
* Person/People - **Persona/Personas**: "personas" is the correct plural for
  "person", but when talking of *people* referring to it as a crowd, we use
  "gente" instead.

## Otros
* You - **Tú**: also "vosotros" if it's in plural.
* We - **Nosotros**
* Message table - **Tablón de mensajes**
* Enter/Intro - **Enter/Intro**
